/*---------------------------------------------------*/
/* Multiway CRVE - bootstrap*/
/* Written by Matt Webb - 08/18/2017 */
/*---------------------------------------------------*/
/* This do file contains Stata programs to calculate */
/* multiway clustered standard errors and */
/* bootstrapped p-values */
/*---------------------------------------------------*/
/* This code should be regarded as in beta version */
/* Please send any bugs or suggestions to:  */
/*  matthewdwebb [at] gmail [dot] com */
/*---------------------------------------------------*/
/*the current code is quite slow for bootstrapping purposes*/
/*---------------------------------------------------*/

set more off
set seed 42

/*example data from Nunn and Wantchekon (2011) - TABLE2.dta*/

clear
import delimited nw.csv

/*-------------------------------------*/
capture program drop MCRVE
program define MCRVE, rclass

	local y "`1'"
	 
	local x "`2'"
	 
	local c "`3'" 

	local g1 "`4'"
	 
	local g2 "`5'"
	
	local pyes "`6'"
		
	/*this is to put in a varlist of controls*/
		local ctrl "${`c'} "
	 	 
	/*cluster by g1 and store the VCE*/
		qui xi: reg `y' `x' `ctrl' , cluster(`g1')
		qui predict uhatg1, resid
		mata: g1 = st_matrix("e(V)")
		local beta = _b[`x']
		mata: temp_beta = `beta'

	/*cluster by g2 and store the VCE*/
		qui xi:reg `y' `x' `ctrl' , cluster(`g2')
		qui predict uhatg2, resid
		mata: g2 = st_matrix("e(V)")

	/*create the intersections*/
		 egen temp_g1 = group(`g1')  if uhatg1 != .
		 
		
		qui egen temp_g2 = group(`g2') if uhatg2 != .
		qui gen temp_g1g2 = 1000000*temp_g1+ temp_g2

	/*cluster by g1g2 and store the VCE*/
		qui xi: reg `y' `x' `ctrl',  cluster(temp_g1g2)
		mata: g1g2 = st_matrix("e(V)")
		qui drop temp_g1g2

	/*combine the two matrices*/
	
		mata: comb = g1 + g2 
	
		mata: MCRVE2 = sqrt(comb[1,1])
		mata: st_numscalar("se2", MCRVE2)
		mata: st_local("se2", strofreal(MCRVE2))
	
	
	/*combine the three matrices*/

		mata: comb = g1 + g2 - g1g2

		mata: MCRVE = sqrt(comb[1,1])

	/*eigenvalue check*/
	mata { 
			/*calculate eigenvalues*/
			B = comb 
			A = makesymmetric(B) 
			symeigensystem(A, C=., lambda=.) 
			
			colnum = cols(lambda)
			
			/*test eigenvalues*/
			for (i=1; i<=colnum; i++) {
				if (lambda[1,i]<0) lambda[1,i]=0
			}
			/*reconstruct matrix*/
			MCRVE_F =C*diag(lambda)*C'
			
			MCRVEF = sqrt(MCRVE_F[1,1])
			
			/*pass this to a local*/
			st_numscalar("se3", MCRVEF)
			mata: st_local("se3", strofreal(MCRVEF))
			
	}
	
	/*calculate the t-stat*/
		mata: T_CRVE3 = temp_beta/MCRVEF
		mata: st_numscalar("T_CRVE3", T_CRVE3)
		mata: st_local("T_CRVE3", strofreal(T_CRVE3))
		
		
		/*calculate the t-stat*/
		mata: T_CRVE2 = temp_beta/MCRVE2
		mata: st_numscalar("T_CRVE2", T_CRVE2)
		mata: st_local("T_CRVE2", strofreal(T_CRVE2))
	
	/*calculate the p-value*/
		local MCRVE_P3 = 999999 /*value if no*/
		local MCRVE_P2 = 999999 /*value if no*/
		
		if "`pyes'" == "yes" {
	
			/*calculate the degrees of freedom*/
			qui summ temp_g1
			local g1 = r(max)
			qui summ temp_g2
			local g2 = r(max)
			
			local dof = min(`g1',`g2')-1
						
			local MCRVE_P3 = 2*min(ttail(`dof',`T_CRVE3'),1-ttail(`dof',`T_CRVE3'))
			
			mata MCRVE_P3 = `MCRVE_P3'
			
			local MCRVE_P2 = 2*min(ttail(`dof',`T_CRVE2'),1-ttail(`dof',`T_CRVE2'))
			
			mata MCRVE_P2 = `MCRVE_P2'
		
		}
		
	qui drop temp_g1 
	qui drop temp_g2
	qui drop uhatg1
	qui drop uhatg2
		
	return scalar t3 = `T_CRVE3'
	return scalar se3 = `se3'
	return scalar p3 = `MCRVE_P3'
	
	return scalar t2 = `T_CRVE2'
	return scalar se2 = `se2'
	return scalar p2 = `MCRVE_P2'
	
	return scalar b = `beta'
	
end /*end of MCRVE program*/
/*---------------------------------------------*/



/*bootstrap program*/
/*---------------------------------------------*/
capture program drop MNWBOOT
program define MNWBOOT, rclass

	local y "`1'"
	 
	local x "`2'"
	 
	local c "`3'" 

	local g1 "`4'"
	 
	local g2 "`5'"
	
	local bs "`6'"
	
	local B = "`7'"
	
	local w = `8'
	
	/*this is to put in a varlist of controls*/
		local ctrl "${`c'} "
	
	/*estimate t-hat */
	MCRVE `y' `x' `c' `g1' `g2' "no"
	local t_hat = `r(t3)'
	mata: T_hat =`t_hat'
	local t_hat2 = `r(t2)'
	mata: T_hat2 =`t_hat2'
	
	
	/*set up bootstrap stuff*/
		cap drop temp_er temp_xbr temp_uni temp_ernew temp_pos temp_ywild
		 qui xi: reg `y' `ctrl'
		
		qui predict temp_er, resid 
		qui predict temp_xbr, xb 
		
		qui gen temp_uni = . 
		qui gen temp_ernew = . 
		qui gen temp_pos = . 
		qui gen temp_ywild = .
				
	/* matrix to store bootstrap statistics */
		mata: T_mnw = J(`B',1,.)
		mata: T_mnw2 = J(`B',1,.)

		
	/*bootstrap loop*/
		forvalues b = 1/`B' {
						
			sort `bs'
			qui by `bs': replace temp_uni = uniform()	
			
			if (`w' == 2) {				
				qui by `bs': replace temp_pos = temp_uni[1]<.5   /*cluster level rademacher indicator*/
				qui replace temp_ernew = (2*temp_pos-1)*temp_er  /*transformed residuals */
				qui replace temp_ywild = temp_xbr + temp_ernew 
			}
			else if (`w' == 6) {
				#delimit ;
				qui by `bs': replace temp_ernew = inrange(temp_uni[1],0,1/6)*temp_er*sqrt(1.5) + inrange(temp_uni[1],1/6,2/6)*temp_er *sqrt(1)+ 
					inrange(temp_uni[1],2/6,3/6)*temp_er*sqrt(0.5) + inrange(temp_uni[1],3/6,4/6)*temp_er*-sqrt(1.5)+
					inrange(temp_uni[1],4/6,5/6)*temp_er*-sqrt(1) + inrange(temp_uni[1],5/6,6/6)*temp_er*-sqrt(0.5);
				# delimit cr
			}
			qui replace temp_ywild = temp_xbr + temp_ernew 
			
			/*estimate and store the MCRVE for the bootstrap sample*/
				MCRVE temp_ywild `x' `c' `g1' `g2' "no"
				local t_star = `r(t3)'
				mata: T_mnw [`b',1]=`t_star'
				local t_star2 = `r(t2)'
				mata: T_mnw2 [`b',1]=`t_star2'
				
		} /*end of bootstrap loop*/
		
		qui cap drop temp_er temp_xbr temp_uni temp_ernew temp_pos temp_ywild
		
		/*calculate the p-value*/
		mata: temp_rej =  abs(T_hat[1,1]):<=abs(T_mnw)
		mata: temp_U = J(rows(temp_rej),1,1) 
		mata: temp_sum = temp_U'*temp_rej
		mata: mnw_p = temp_sum / rows(temp_rej)
		mata: mnw_p = max(mnw_p\ 0)
		
		mata: st_numscalar("mnw_p", mnw_p)
		mata: st_local("mnw_p", strofreal(mnw_p))
		
		return scalar p = `mnw_p'
		
		/*calculate the p-value 2-matrix*/
		mata: temp_rej =  abs(T_hat2[1,1]):<=abs(T_mnw2)
		mata: temp_U = J(rows(temp_rej),1,1) 
		mata: temp_sum = temp_U'*temp_rej
		mata: mnw_p2 = temp_sum / rows(temp_rej)
		mata: mnw_p2 = max(mnw_p2\ 0)
		
		mata: st_numscalar("mnw_p2", mnw_p2)
		mata: st_local("mnw_p2", strofreal(mnw_p2))
		
		return scalar p2 = `mnw_p2'
	
end
/*---------------------------------------------*/

/*call the program(s) to calculate the standard error and p-value*/

	/* these arguments below can be modified by the user*/
	
	local y "trust_neighbors"  /*outcome variable*/
		 
	local x "exports"  /*variable to calculate p-values for*/
		 
	/*list all control variables here*/
	global CTRL "age age2 male urban_dum edc* occp* rlg* lvg* ctry* district_ethnic_frac frac_ethnicity_in_district"
	local c "CTRL"
	
	local g1 "eth" /*first clustering dimension*/
		 
	local g2 "country" /*second clustering dimension*/
	
	local bs "country"  /*bootstrap by - variable*/
	
	local B = 999 /*number of bootstraps - e.g. 999*/
	
	local w = 2 /*weight either 2 or 6*/
		
	*MNWBOOT `y' `x' `c' `g1' `g2' `bs' `B' `w'	
	*disp "the bootstrap-3 p-value is `r(p)'"
	*disp "the bootstrap-2 p-value is `r(p2)'"
	
	/*this will calculate the multiway-CRVE for the above*/
	
	*local pval "yes"  /*set to yes of no*/
	/*
	MCRVE `y' `x' `c' `g1' `g2' "`pval'"
	disp "the MCRVE3 p-value is `r(p3)' "
	disp "the MCRVE3 t-stat is `r(t3)' "
	disp "the MCRVE3 se is `r(se3)' "
	disp "the MCRVE2 p-value is `r(p2)' "
	disp "the MCRVE2 t-stat is `r(t2)' "
	disp "the MCRVE2 se is `r(se2)' "
	disp "the MCRVE beta is `r(b)' "
	
	*/
	
	
/*---------------------------------------------------------*/
/*to generate the ALL multiway CRVE and bootstrap p-values */
/* from the empirical example in Table 3 run the code below.   */
/* With B=9999 and a single core version of Stata this will */
/* take several days to runwith the current verison of     */
/* the code. */
/*---------------------------------------------------------*/
/* */

/*set bootstrap number - table uses B=9999*/	
	local B = 9999

/*generate a unique identifier WR bootstrap*/
	gen temp_ind = _n


/* clustering levels*/
	global CLUSTVARS "varlist dstr country"
	
foreach cvar of $CLUSTVARS {
	
		
	/*estimate the M-CRVE p-value*/
		disp "cluster by " "`cvar'"
		
		reg `y' `x' ${CTRL}, cluster(`cvar')
		reg `y' `x' ${CTRL}, cluster(eth)	
		reg `y' `x' ${CTRL}, cluster(country)
	
	
		MCRVE `y' `x' `c' `g1' `cvar' "yes"
		disp "the MCRVE3 se is `r(se3)' "
		disp "the MCRVE 3-matrix p-value is `r(p3)' "
		disp "the MCRVE2 se is `r(se2)' "
		disp "the MCRVE 2-matrix p-value is `r(p2)' "
		
		
		/*generate a unique identifier for intersection*/
		qui egen temp_g1 = group(eth)
		qui egen temp_g2 = group(`cvar')
		qui gen temp_int = 1000000*temp_g1+ temp_g2
		qui cap drop temp_g1 temp_g2
		
	/*loop over bootstrap levels*/	
	/**/
	/* bootstrap levels* - WCR-geog WCR-eth  WCR-intersect  WR */
		global BOOTSVARS "varlist `cvar' eth  temp_int temp_ind "
		foreach bs of $BOOTSVARS {
			disp "bootstrap by " "`bs'"
			
			/*estimate the bootstrap p-values*/
			capture log close
			log using "${PATH}log_`cvar'_`bs'_1.txt", text replace
			disp "bootstrap by " "`bs'"
			disp "cluster by " "`cvar'"
			
			set seed 424242
			
			MNWBOOT `y' `x' `c' `g1' `cvar' `bs' `B' `w'
			disp "the 3-matrix bootstrap p-value is `r(p)'"
			disp "the 2-matrix bootstrap p-value is `r(p2)'"
			capture log close
			
		}

		cap drop temp_int
		
		/* */
} /*end of c-var*/

/**/
		
/*version history */
*1 - 08/18/2017 - initial version
*2 - 02/02/2019 - two matrix versions included 
*3 - 03/21/2019 - fixed how DOF are determined	
